# Codebase Underlying the BSc Thesis: Type-Checking Modules and Imports using Scope Graphs

The objective of this research was to determine whether a stratified type-checking approach using scope graphs could type-check the proof-of-concept language LM. Scope graphs provide a way to type-check real-world programming languages and their constructs. Previous implementations that type-check LM, a language with relative, unordered, and glob imports, do not halt. This dataset contains Haskell code for constructing and type-checking a scope graph of an LM program. It is based on the [Phased Haskell library](https://github.com/MetaBorgCube/scope-graph-scheduling-bsc-template/tree/cfa6ab83d50797e861587d0af626fc10765741da). With this implementation, the all test cases halt and the majority exhibit the correct behaviour with only one false-negative.

This implementation uses a five-step approach:

1. Constructing a module hierarchy.
2. Constructing a scope graph consisting of scopes and module sinks.
3. Iteratively resolving imports and placing import edges in the scope graph.
4. Adding all declarations of all modules to the scope graph.
5. Type-checking the bodies of all declarations with respect to the scope graph.


Many test cases bundled with this data set are based on those for [LMR](https://github.com/MetaBorgCube/metaborg-lmr/tree/d1cb439be8eba9468d95cfc0aa091558147a227d/lang.lmr.mstx/tests) (which is very similar to LM) and can be found here. On top of that. more test cases were derived and included. All test cases are denoted as annotated terms.

This data set is linked to a Bachelor's thesis completed at the EEMCS faculty at the TU Delft. A link will be added after publication.

## Introduction

This codebase is written in [Haskell](https://www.haskell.org/), the files are provided in the form of a [Cabal](https://www.haskell.org/cabal/) project. Included are both a tarballed (`lmtc.tar.gz`) archive of this project, as well as a Git repository that may be cloned to access the files. 

The structure is as follows:
- The type-checker itself is in the library module `lang-lmr/lib/`.
- A test suite is in `lang-lmr/tests/`, which runs the tests specified in `aterms-res/lmr/`.
- The implementation can run the test cases by executing `cabal test lmr-test` from the root of the project files.

## Methodological Information

The data itself is a codebase, it consists of incremental commits to create the software. The code was based on a [template](https://github.com/MetaBorgCube/scope-graph-scheduling-bsc-template/tree/cfa6ab83d50797e861587d0af626fc10765741da). This template contained library code to facilitate creating the type-checker.

This codebases also contains example programs, inherited from the template.
The LM code snippets were derived from the [Mini-Statix implementation](https://github.com/MetaBorgCube/metaborg-lmr/tree/d1cb439be8eba9468d95cfc0aa091558147a227d/lang.lmr.mstx/tests) and heavily augmented with more examples that came up during this research.

It was developed using *GHC 8.10.7* and *Cabal3.6.2.0* on both a Unix and Windows environment, though the latter sometimes has encoding issues with terminal output. Using macOS or Linux with those versions of the technology should compile the codebase and run the test suite (`cabal test lmr-test`).

## Licence

The code is available under the MIT licence.

